/* Fichier "common_geometry.c"
 * contient diverses fonctions utilitaires qui grent 
 * les operations de base sur les vecteurs et les triangles
 *
 * Attention, vecteor_3d ne designe pas forcement un vecteur :
 * ca peut aussi etre un point (i.e. un vecteur avec (0,0,0) comme 
 * origine.
 * cf. les description des fct en cas d'ambiguit
 */

#include "projet.h"

/* ======================================================
 * set_vector
 *
 * Description: met  jour les coordonnes d'un vecteur 
 * Parametres: pointeur sur le vecteur  modifier et les 
 *  3 nouvelles coordonnees  
 * =====================================================
 */
void set_vector(vector3d_t *vector, float x, float y, float z)
{
  vector->coord[0] = x;
  vector->coord[1] = y;
  vector->coord[2] = z;  
}

/* ======================================================
 * cross_product
 *
 * Description: effectue le produit vectoriel de deux vecteurs
 *  v1 et v2
 * Parametres: les 2 vecteurs
 * Renvoie: le vecteur resultat de l'operation
 * =====================================================
 */
vector3d_t cross_product_vect(vector3d_t v1, vector3d_t v2)
{
  vector3d_t ret;
  
  float *coord1 = v1.coord;
  float *coord2 = v2.coord;
   
  /* [v1-v2] cross [v2-v3] */
  ret.coord[0] = coord1[1]*coord2[2] - coord2[1]*coord1[2];
  ret.coord[1] = coord2[0]*coord1[2] - coord1[0]*coord2[2];
  ret.coord[2] = coord1[0]*coord2[1] - coord2[0]*coord1[1];
  
  return ret;
}


/* ======================================================
 * cross_product
 *
 * Description: effectue le produit vectoriel de deux vecteurs
 *  AB et AC definis par 3 points A, B, C
 * Parametres: les 3 points A, B et C
 *  Attention, v1, v2 et v3 sont des POINTS d'o leur nombre (3)
 * Renvoie: le _vecteur_ resultat de l'operation
 * =====================================================
 */
vector3d_t cross_product(vector3d_t A, vector3d_t B, vector3d_t C)
{
  float n1[3];
  float n2[3];
  vector3d_t ret;
  
  float *coord1 = A.coord;
  float *coord2 = B.coord;
  float *coord3 = C.coord;

#if 0
  /* vect. AB   (B - A) */
  n1[0] = coord2[0]-coord1[0];
  n1[1] = coord2[1]-coord1[1];
  n1[2] = coord2[2]-coord1[2];
  
  /* vect AC (C - A) */
  n2[0] = coord3[0]-coord1[0];
  n2[1] = coord3[1]-coord1[1];
  n2[2] = coord3[2]-coord1[2];
#endif 
 
  /* A - B */
  n1[0] = coord1[0]-coord2[0];
  n1[1] = coord1[1]-coord2[1];
  n1[2] = coord1[2]-coord2[2];
			
  /* B - C */
  n2[0] = coord2[0]-coord3[0];
  n2[1] = coord2[1]-coord3[1];
  n2[2] = coord2[2]-coord3[2];
  
  /* AB vectoriel AC */
  ret.coord[0] = n1[1]*n2[2] - n2[1]*n1[2];
  ret.coord[1] = n2[0]*n1[2] - n1[0]*n2[2];
  ret.coord[2] = n1[0]*n2[1] - n2[0]*n1[1];
  return ret;
}

float
get_vect_length(vector3d_t v)
{
  return sqrt( v.coord[0]*v.coord[0] + v.coord[1]*v.coord[1] 
     + v.coord[2]*v.coord[2]);
}

/* ======================================================
 * normalize
 *
 * Description: divise un vecteur par sa norme
 * Parametres: pointeur sur le vecteur  modifier
 * =====================================================
 */
void normalize(vector3d_t *v)
{
  float length;
  
  /* v / |v| */
  length = get_vect_length(*v);
  
  v->coord[0] /= length;
  v->coord[1] /= length;
  v->coord[2] /= length; 
}

/* ======================================================
 * set_triangle
 *
 * Description: met  jour les coord. de chacun des 3 points
 *  d'un triangle
 * Parametres: pointeur sur le triangle  modifier et les
 *   3 points
 * =====================================================
 */
void set_triangle(triangle3d_t *triangle, vector3d_t v1, vector3d_t v2,
      vector3d_t v3)
{
  triangle->vertices[0] = v1;
  triangle->vertices[1] = v2;
  triangle->vertices[2] = v3;
  
  triangle->normal = cross_product(v1, v2, v3);
  normalize(&triangle->normal);
}
