/*  Fichier "gl_generic.c"
 *  contient les fonctions appeles par la librairie
 *  qui gre l'affichage et les eventuels widgets.
 *  Ces fonctions sont gnriques, i.e. ne _dpendent pas_ de 
 *  la librairie utilise.
 *   
 *  Ceci permet de pouvoir changer Gtk+ avec GLUT ou tout autre toolkit
 *  trs facilement, en rutilisant ces fonctions gnriques.
 *  Le but est d'ameliorer la portabilit, jute au cas o...
 *  (soyons prvoyants)
 */

#include "projet.h"
int wireframe = 0;
int bfculling = 1;

/* ======================================================
 * gl_generic_resize
 *
 * Description:  appeler quand la fenetre de rendu 
 *  est redimensionne
 * Parametres: w: nouvelle largeur, h: nouvelle hauteur	
 * =====================================================
 */
void
gl_generic_resize (GLfloat w, GLfloat h)
{
  glViewport (0, 0, w, h);

  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
  gluPerspective (60.0, w / h, 1.0f, 4000.0f);

  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity ();
}

/* ======================================================
 * gl_generic_init
 *
 * Description: initialisation d'OpenGL
 * =====================================================
 */
void
gl_generic_init (void)
{
  GLfloat lmodel_ambient[] = { 0.4, 0.4, 0.4, 0.4 };
  GLfloat light_position[] = { 0.0, 200.0, 0.0, 1.0 };
  GLfloat white_light[] = { 0.8, 0.8, 0.8, 0.8 };

  glEnable (GL_DEPTH_TEST);
  /*glEnable(GL_TEXTURE_2D); */
  glDisable(GL_TEXTURE_2D);

  glFrontFace (GL_CCW);
  glCullFace (GL_BACK);
  glEnable (GL_CULL_FACE);

  glEnable (GL_NORMALIZE);

  glClearColor (0.0, 0.0, 0.0, 0.0);

  glEnable (GL_LIGHTING);
  glLightModeli (GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);

  glShadeModel (GL_SMOOTH);

  glLightModelfv (GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  glLightfv (GL_LIGHT0, GL_POSITION, light_position);
  glLightfv (GL_LIGHT0, GL_DIFFUSE, white_light);
  glLightfv (GL_LIGHT0, GL_SPECULAR, white_light);
  glEnable (GL_LIGHT0);

  gl_generic_resize (800, 600);
  
  cam_set_default_view ();
}

void 
gl_generic_init_2D_intro (void)
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, 800, 0, 600, -1.0, 1.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glColor4f(1.0, 1.0, 1.0, 1.0);
  
  glEnable( GL_TEXTURE_2D );
  glDisable( GL_DEPTH_TEST );
  glDisable( GL_CULL_FACE );
  glDisable( GL_LIGHTING );
  glDisable( GL_NORMALIZE );
  
  glintro_load_2D_intro ();
}

/* ======================================================
 * gl_generic_render_scene
 *
 * Description: rendu de la scene en appelant la 
 *  display list du terrain
 * =====================================================
 */
void
gl_generic_render_scene (void)
{
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glLoadIdentity ();

  cam_look ();

  if (terrain->generated) {
    glPushMatrix ();
    glCallList (terrain->displaylist);
    glPopMatrix ();
  }
}

void
gl_generic_switch_wireframe (void)
{
  wireframe = !wireframe;

  if (wireframe)
    glPolygonMode (GL_FRONT, GL_LINE);
  else
    glPolygonMode (GL_FRONT, GL_FILL);
}

void
gl_generic_switch_bfculling (void)
{
  bfculling = !bfculling;

  if (bfculling)
    glEnable (GL_CULL_FACE);
  else
    glDisable (GL_CULL_FACE);
}
