#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>

#include "terrain.h"

#define LN_2 0.693147181f

void
midpoint_disp_iterate (terrain_t * terrain, float rmax, int compteur,
		       int max_iter,
		       int i_a, int j_a, int i_b, int j_b,
		       int i_c, int j_c, int i_d, int j_d)
{
  int i_e, j_e, i_f, j_f, i_g, j_g, i_h, j_h, i_m, j_m;
  float z_e, z_f, z_g, z_h, z_m;	/* altitudes */
  float r;
  param_middisp_t *param;
  param = (param_middisp_t *) terrain->spec_param;
  float h = param->h;

  i_e = i_a + (i_d - i_a) / 2;
  j_e = j_a;

  i_f = i_a;
  j_f = j_a + (j_b - j_a) / 2;

  i_g = i_b + (i_c - i_b) / 2;
  j_g = j_b;

  i_h = i_c;
  j_h = j_d + (j_c - j_d) / 2;

  i_m = i_e;
  j_m = j_f;

#ifdef DEBUG_MIDPOINT
  printf ("A=(%d,%d), B=(%d,%d), C=(%d,%d), D=(%d,%d)\n",
	  i_a, j_a, i_b, j_b, i_c, j_c, i_d, j_d);

  printf ("E=(%d,%d), F=(%d,%d), G=(%d,%d), H=(%d,%d)\n",
	  i_e, j_e, i_f, j_f, i_g, j_g, i_h, j_h);
#endif

  r = random_float (-rmax, rmax);
  z_e = (get_heightmap_value (terrain, i_a, j_a)
	 + get_heightmap_value (terrain, i_d, j_d)) / 2 + (r);

  r = random_float (-rmax, rmax);
  z_f = (get_heightmap_value (terrain, i_a, j_a)
	 + get_heightmap_value (terrain, i_b, j_b)) / 2 + (r);

  r = random_float (-rmax, rmax);
  z_g = (get_heightmap_value (terrain, i_b, j_b)
	 + get_heightmap_value (terrain, i_c, j_c)) / 2 + (r);

  r = random_float (-rmax, rmax);
  z_h = (get_heightmap_value (terrain, i_c, j_c)
	 + get_heightmap_value (terrain, i_d, j_d)) / 2 + (r);

  r = random_float (-rmax, rmax);
  z_m = (get_heightmap_value (terrain, i_a, j_a)
	 + get_heightmap_value (terrain, i_b, j_b)
	 + get_heightmap_value (terrain, i_c, j_c)
	 + get_heightmap_value (terrain, i_d, j_d)) / 4 + (r);

  set_heightmap_value (terrain, i_e, j_e, z_e);
  if (z_e > terrain->max_height)
    terrain->max_height = z_e;

  set_heightmap_value (terrain, i_f, j_f, z_f);
  if (z_f > terrain->max_height)
    terrain->max_height = z_f;

  set_heightmap_value (terrain, i_g, j_g, z_g);
  if (z_g > terrain->max_height)
    terrain->max_height = z_g;

  set_heightmap_value (terrain, i_h, j_h, z_h);
  if (z_h > terrain->max_height)
    terrain->max_height = z_h;

  set_heightmap_value (terrain, i_m, j_m, z_m);
  if (z_m > terrain->max_height)
    terrain->max_height = z_m;


  rmax = rmax * (float) (pow (2.0f, -h));
  compteur++;

  if (compteur < max_iter) {

#ifdef DEBUG_MIDPOINT
    printf ("square0\n");
#endif
    midpoint_disp_iterate (terrain, rmax, compteur, max_iter,
			   i_a, j_a, i_f, j_f, i_m, j_m, i_e, j_e);
#ifdef DEBUG_MIDPOINT
    printf ("square1\n");
#endif
    midpoint_disp_iterate (terrain, rmax, compteur, max_iter,
			   i_f, j_f, i_b, j_b, i_g, j_g, i_m, j_m);
#ifdef DEBUG_MIDPOINT
    printf ("square2\n");
#endif
    midpoint_disp_iterate (terrain, rmax, compteur, max_iter,
			   i_m, j_m, i_g, j_g, i_c, j_c, i_h, j_h);

#ifdef DEBUG_MIDPOINT
    printf ("square3\n");
#endif
    midpoint_disp_iterate (terrain, rmax, compteur, max_iter,
			   i_e, j_e, i_m, j_m, i_h, j_h, i_d, j_d);
  }

  else {
#ifdef DEBUG_MIDPOINT
    printf ("j'en ai marre\n");
#endif
    return;
  }
}

void
gen_terrain_middisp (terrain_t * terrain)
{
  int N;
  float m;
  param_middisp_t *param;
  param = (param_middisp_t *) terrain->spec_param;
  m = (param->factor / 100.0f);
  
  srand (time (NULL));

  N = (int) rintf ((float) (log (terrain->size - 1)) / LN_2);

  midpoint_disp_iterate (terrain, 20.0f, 0, N,
			 0, 0, 0, terrain->size - 1,
			 terrain->size - 1, terrain->size - 1,
			 terrain->size - 1, 0);

  TER_FIL_multiply (terrain, m);
  
  printf ("MIDDISP: max_height=%f\n", terrain->max_height);
}
