#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "terrain.h"

float
get_heightmap_value (terrain_t * terrain, int i, int j)
{
  if (i < 0 || i >= terrain->size || j < 0 || j >= terrain->size) {
    fprintf (stderr, "Warning: out of range in get_heightmap_value (request:(%d,%d)\n",
    i,j);
    return 0;
  }
  else
    return (terrain->heightmap[(j * terrain->size) + i]);
}

void
set_heightmap_value (terrain_t * terrain, int i, int j, float value)
{
  if (i < 0 || i >= terrain->size || j < 0 || j >= terrain->size) {
    fprintf (stderr, "Warning: out of range in set_heightmap_value (request:(%d,%d)\n",
    i,j);
  }
  else
    terrain->heightmap[(j * terrain->size) + i] = value;
}

float
random_float (float min, float max)
{
  int r;
  float x;

  r = rand ();

  /* on normalise entre 0 et 1 */
  x = (float) (r & 0x7fff) / (float) 0x7fff;

  /* et on fait un petit chgt d'echelle puis translation */
  return (x * (max - min) + min);
}

/* renvoie un entier entre 0 et max-1 */
int
random_int (int max)
{
  int r;

  r = rand ();

  return (r % max);
}

void
reset_filters (terrain_t * terrain)
{
  int i=FILTER1;
  
  for (i=0; i < NUM_EFFECTIVE_FILTERS; i++)  {
    terrain->filters[i].name = TERR_FILTER_NOTHING;
    terrain->filters[i].param = 0.0f;
  }
}

void
TER_update_maxheight_value (terrain_t *terrain)
{
  int i=0, j=0;
  float z=0;
  
  for (i = 0; i < terrain->size; i++) {
    for (j = 0; j < terrain->size; j++) {
      z = get_heightmap_value (terrain, i, j);
      if (z > terrain->max_height)
        terrain->max_height = z;
    }
  }
}

char
create_new_terrain (terrain_t ** terrain)
{
  *terrain = calloc (1, sizeof (terrain_t));
  if (*terrain == NULL)
    return 0;

  (*terrain)->size = DEFAULT_TERR_SIZE;
  (*terrain)->max_height = 0.0f;

  (*terrain)->heightmap = NULL;
  (*terrain)->heightmap_orig = NULL;
  (*terrain)->spec_param = NULL;
  (*terrain)->generated = 0;
  (*terrain)->algo = DEFAULT_TERR_ALGO;
  (*terrain)->color_scheme = DEFAULT_TERR_COLOR;
  
  srand (time (NULL));

  return 1;
}

char
init_heightmap (terrain_t * terrain)
{
  if (terrain->heightmap) {
    /*fprintf (stderr, "Erasing existing terrain\n");*/
    free (terrain->heightmap);
  }

  terrain->heightmap_orig = (float *) calloc (terrain->size * terrain->size,
					 sizeof (float));
  terrain->heightmap = (float *) calloc (terrain->size * terrain->size,
					 sizeof (float));

  if (terrain->heightmap == NULL || terrain->heightmap_orig == NULL)
    return 0;

#define FIXME
#warning FIX ME
  
#ifndef FIXME
  if (terrain->spec_param == NULL) {
    fprintf (stderr, "Error: no terrain parameters found.\n");
    return 0;
  }
#endif
  reset_filters(terrain);
  
  terrain->max_height = 0.0f;
  terrain->generated = 0;

  return 1;
}

void
copy_orig_terr_to_effective (terrain_t *terrain)
{
  int i=0;
  for (i=0; i < terrain->size * terrain->size; i++) {
    terrain->heightmap[i] = terrain->heightmap_orig[i];  
  }
}

void
copy_effective_terr_to_orig (terrain_t *terrain)
{
  int i=0;
  for (i=0; i < terrain->size * terrain->size; i++) {
    terrain->heightmap_orig[i] = terrain->heightmap[i];  
  }
}

char
gen_terrain (terrain_t * terrain)
{
  if (init_heightmap (terrain)) {

    switch (terrain->algo) {
    case TERR_FLAT:
      break;
    case TERR_MIDDISP:
      gen_terrain_middisp (terrain);
      break;
    case TERR_SPHERE:
      gen_terrain_sphere (terrain);
      break;
    case TERR_HYPERB:
      gen_terrain_hyperbole (terrain);
      break;
    case TERR_DIAM:
      gen_terrain_diamond(terrain);
      break;

    default:
      fprintf (stderr, "Warning: unknown algorithm\n");
      return 0;
      break;
    }
    
    copy_effective_terr_to_orig(terrain);
    
    /* on ne garde que les valeurs au dessus du niveau de la mer */
    TER_FIL_allpositive(terrain);
    
    return 1;
  }

  else
    return 0;
}

char
apply_filter (terrain_t * terrain, int filter, float param)
{
  if (terrain->generated) {

    switch (filter) {
    case TERR_FILTER_NOTHING:
      break;
    case TERR_FILTER_FIR:
      TER_FIL_FIR(terrain, param);
      break;
    case TERR_FILTER_PUISS:
      TER_FIL_power(terrain,param);
      break;
    case TERR_FILTER_RIV:
      TER_FIL_river(terrain,(int)param);
      break;
    case TERR_FILTER_CRAT:
      TER_FIL_craters (terrain, param);
      break;
    default:
      fprintf (stderr, "Warning: unknown filter\n");
      return 0;
      break;
    }
    return 1;
  }

  else
    return 0;
}
