/* Fichier "export_bmp.c"
 * Exportation du terrain en heightfield au format bmp
 *
 * il faut dfinir USE_BMP_EXPORT pour compiler ce fichier, parce qu'il
 * ne sert  rien pour le projet (je ne me suis apercu que POV 
 * n'acceptait pas le format BMP pour les heightfield qu'_aprs_ avoir 
 * crit ce code... on le garde au cas o ca serve un jour, maintenant
 * que c'est l...)
 */
 
#ifdef USE_BMP_EXPORT
 
#include <stdio.h>
#include <stdlib.h>

#include <glib.h>

#include "terrain.h"

typedef struct BmpHeader BmpHeader;

struct BmpHeader
{
  guint32 bfSize;
  guint32 bfReserved;
  guint32 bfOffBits;
  guint32 biSize;
  guint32 biWidth;
  guint32 biHeight;
  guint16 biPlanes;
  guint16 biBitCount;
  guint32 biCompression;
  guint32 biSizeImage;
  guint32 biXPelsPerMeter;
  guint32 biYPelsPerMeter;
  guint32 biClrUsed;
  guint32 biClrImportant;
};

int terr_export_bmp_bw (terrain_t *terrain, char *filename)
{
  FILE      *out;
  gint       row_size, row_width;
  BmpHeader  header;
  gint       x, y;
  gint       status;

  out = fopen (filename, "wb");
  if (!out)
    return -1;

  row_size = terrain->size;
  row_width = (row_size + 3) & 0xfffc;

  header.bfSize = GUINT32_TO_LE (54 + 256 * 4 + row_width * terrain->size);
  header.bfReserved = 0;
  header.bfOffBits = GUINT32_TO_LE (54 + 256 * 4);
  header.biSize = GUINT32_TO_LE (40);
  header.biWidth = GUINT32_TO_LE (terrain->size);
  header.biHeight = GUINT32_TO_LE (terrain->size);
  header.biPlanes = GUINT16_TO_LE (1);
  header.biBitCount = GUINT16_TO_LE (8);
  header.biCompression = 0;
  header.biSizeImage = GUINT32_TO_LE (row_width * terrain->size);
  header.biXPelsPerMeter = GUINT32_TO_LE (2925);
  header.biYPelsPerMeter = GUINT32_TO_LE (2925);
  header.biClrUsed = 256;
  header.biClrImportant = 256;

  fwrite ("BM", 2, 1, out);
  fwrite (&header, sizeof (header), 1, out);

  for (y = 0; y < 256; y++)
    {
      for (x = 0; x < 3; x++)
        fputc (y, out);

      fputc ('\0', out);
    }

  for (y = 0; y < terrain->size; y++)
    {
      gfloat *pos;

      pos = &terrain->heightmap[(terrain->size - y - 1) * terrain->size];
      for (x = 0; x < terrain->size; x++)
        {
          gint32 value;

          value = (gint32) (MAX (MIN ( ((*pos)/terrain->max_height) * 255.0, 255.0), 0.0) + 0.5);
          fputc (value, out);

          pos++;
        }

      for (; x < row_width; x++)
        fputc ('\0', out);
    }

  status = ferror (out);
  fclose (out);

  return status;
}

#endif
