#include <stdlib.h>
#include <stdio.h>

#ifndef DONT_HAVE_ZLIB
 #include <zlib.h>
#endif

#include "terrain.h"

char TERR_HEADER_VERIF[3] = { 'T', 'E', 'R' };

#ifndef DONT_HAVE_ZLIB
void
TER_write_hyperb_params (terrain_t * terrain, gzFile file)
{
  param_hyperb_t *param = (param_hyperb_t *) terrain->spec_param;

  gzwrite (file, &param->power, sizeof (double) * 1);
  gzwrite (file, &param->factor, sizeof (float) * 1);
  gzwrite (file, &param->num_iter, sizeof (int) * 1);
  gzwrite (file, &param->radius_max, sizeof (int) * 1);
  gzwrite (file, &param->radius_min, sizeof (int) * 1);
  gzwrite (file, &param->epsilon_type, sizeof (int) * 1);
}

void
TER_read_hyperb_params (terrain_t * terrain, param_hyperb_t * param, gzFile file)
{
  if (param) 
  {
    gzread (file, &param->power, sizeof (double) * 1);
    gzread (file, &param->factor, sizeof (float) * 1);
    gzread (file, &param->num_iter, sizeof (int) * 1);
    gzread (file, &param->radius_max, sizeof (int) * 1);
    gzread (file, &param->radius_min, sizeof (int) * 1);
    gzread (file, &param->epsilon_type, sizeof (int) * 1);
  }
  
  else fprintf(stderr,"ERROR: no parameters structure supplied\n");
}

void
TER_write_middisp_params (terrain_t * terrain, gzFile file)
{
  param_middisp_t *param = (param_middisp_t *) terrain->spec_param;

  gzwrite (file, &param->h, sizeof (float) * 1);
  gzwrite (file, &param->factor, sizeof (float) * 1);
}

void
TER_read_middisp_params (terrain_t * terrain, param_middisp_t * param, gzFile file)
{
  if (param) 
  {
    gzread (file, &param->h, sizeof (float) * 1);
    gzread (file, &param->factor, sizeof (float) * 1);
  }
  
  else fprintf(stderr,"ERROR: no parameters structure supplied\n");
}

void
TER_write_diamsquare_params (terrain_t * terrain, gzFile file)
{
  param_diamsquare_t *param = (param_diamsquare_t *) terrain->spec_param;

  gzwrite (file, &param->h, sizeof (float) * 1);
  gzwrite (file, &param->factor, sizeof (float) * 1);
  gzwrite (file, &param->wrap, sizeof (int) * 1);
}

void
TER_read_diamsquare_params (terrain_t * terrain, param_diamsquare_t * param, gzFile file)
{
  if (param) 
  {
    gzread (file, &param->h, sizeof (float) * 1);
    gzread (file, &param->factor, sizeof (float) * 1);
    gzread (file, &param->wrap, sizeof (int) * 1);
  }
  
  else fprintf(stderr,"ERROR: no parameters structure supplied\n");
}

int
TER_export_terrain (terrain_t * terrain, char *filename)
{
  gzFile file;
  int i=0;

  if (!terrain)
    return 0;

  if (!(file = (gzFile)gzopen(filename, "wb9")))
  {
    fprintf(stderr, "Error creating file: %s.\n", filename);
    return 0;
  }

  /* header pour verification de format lors d'une lecture future */
  gzwrite (file, TERR_HEADER_VERIF, sizeof (char) * 3);

  gzwrite (file, &terrain->algo, sizeof (int) * 1);
  gzwrite (file, &terrain->size, sizeof (int) * 1);
  
  gzwrite (file, &terrain->color_scheme, sizeof (int) * 1);

  /* parametres specifiques au type d'algo */
  switch (terrain->algo) {
  case TERR_FLAT:
    break;
  case TERR_MIDDISP:
    TER_write_middisp_params (terrain, file);
    break;
  case TERR_SPHERE:
    break;
  case TERR_HYPERB:
    TER_write_hyperb_params (terrain, file);
    break;
  case TERR_DIAM:
    TER_write_diamsquare_params (terrain, file);
    break;

  default:
    fprintf (stderr, "Warning: unknown algorithm\n");
    break;
  }

  /* terrain en lui meme */
  gzwrite (file, terrain->heightmap_orig,
	  sizeof (float) * terrain->size * terrain->size);
  gzwrite (file, terrain->heightmap,
	  sizeof (float) * terrain->size * terrain->size);
  gzwrite (file, &terrain->max_height, sizeof (float) * 1);
  
  /* filtres */
  for (i=0; i < NUM_EFFECTIVE_FILTERS; i++) {
    gzwrite (file, &terrain->filters[i].name, sizeof (int) * 1);
    gzwrite (file, &terrain->filters[i].param, sizeof (float) * 1);
  }

  gzclose(file);
  return 1;
}

int
TER_verify_terrain_file (gzFile file)
{
  char verification[3];
  if (!file) return 0;
    
  gzread(file, verification, sizeof(char)*3);
  if ((verification[0] != TERR_HEADER_VERIF[0]) 
      || (verification[1] != TERR_HEADER_VERIF[1])
      || (verification[2] != TERR_HEADER_VERIF[2]))   
    return 0;

  else return 1;
}

int
TER_read_algo (char *filename)
{
  gzFile file;
  int algo;
  
  if (!(file = (gzFile)gzopen(filename, "rb9")))
  {
    fprintf(stderr, "Error opening file: %s.\n", filename);
    return -1;
  }
  
  if (!TER_verify_terrain_file (file)) {
    fprintf(stderr, "Error: file is not a terrain\n");
    gzclose(file);
    return -1;
  }
  
  gzread (file, &algo, sizeof (int) * 1);  
  gzclose (file);
  
  return algo;
}

int
TER_import_terrain (terrain_t * terrain, void *param, char *filename)
{
  gzFile file;
  int i=0;
  
#warning FIX ME  
  if (!terrain)
  /*if (!terrain || !param)*/
    return 0;
  
  if (!(file = (gzFile)gzopen(filename, "rb9")))
  {
    fprintf(stderr, "Error opening file: %s.\n", filename);
    return 0;
  }
  
  if (!TER_verify_terrain_file (file)) {
    fprintf(stderr, "Error: file is not a terrain\n");
    gzclose(file);
    return 0;
  }

  gzread (file, &terrain->algo, sizeof (int) * 1);
  gzread (file, &terrain->size, sizeof (int) * 1);
  
  gzread (file, &terrain->color_scheme, sizeof (int) * 1);
  
  /* alloue la memoire pour les heightmaps */
  if (!init_heightmap (terrain)) {
    fprintf(stderr,"Error: error while initializing heightmaps\n");
    return 0;
  }
  
  /* parametres specifiques au type d'algo */
  switch (terrain->algo) {
  case TERR_FLAT:
    break;
  case TERR_MIDDISP:
    TER_read_middisp_params (terrain, param, file);
    break;
  case TERR_SPHERE:
    break;
  case TERR_HYPERB:
    TER_read_hyperb_params (terrain, param, file);
    break;
  case TERR_DIAM:
    TER_read_diamsquare_params (terrain, param, file);
    break;

  default:
    fprintf (stderr, "Warning: unknown algorithm\n");
    break;
  }
  
  terrain->spec_param = (void *) param;

  /* terrain en lui meme */
  gzread (file, terrain->heightmap_orig,
	  sizeof (float) * terrain->size * terrain->size);
  gzread (file, terrain->heightmap,
	  sizeof (float) * terrain->size * terrain->size);
  gzread (file, &terrain->max_height, sizeof (float) * 1);
  
  /* filtres */
  for (i=0; i < NUM_EFFECTIVE_FILTERS; i++) {
    gzread (file, &terrain->filters[i].name, sizeof (int) * 1);
    gzread (file, &terrain->filters[i].param, sizeof (float) * 1);
  }
  
  gzclose(file);
  return 1;
}

#else /* import/export dans compression (pas d'utilisation de zlib) */
void
TER_write_hyperb_params (terrain_t * terrain, FILE * file)
{
  param_hyperb_t *param = (param_hyperb_t *) terrain->spec_param;

  fwrite (&param->power, sizeof (double), 1, file);
  fwrite (&param->factor, sizeof (float), 1, file);
  fwrite (&param->num_iter, sizeof (int), 1, file);
  fwrite (&param->radius_max, sizeof (int), 1, file);
  fwrite (&param->radius_min, sizeof (int), 1, file);
  fwrite (&param->epsilon_type, sizeof (int), 1, file);
}

void
TER_read_hyperb_params (terrain_t * terrain, param_hyperb_t * param, FILE * file)
{
  fread (&param->power, sizeof (double), 1, file);
  fread (&param->factor, sizeof (float), 1, file);
  fread (&param->num_iter, sizeof (int), 1, file);
  fread (&param->radius_max, sizeof (int), 1, file);
  fread (&param->radius_min, sizeof (int), 1, file);
  fread (&param->epsilon_type, sizeof (int), 1, file);
}

void
TER_write_middisp_params (terrain_t * terrain, FILE * file)
{
  param_middisp_t *param = (param_middisp_t *) terrain->spec_param;

  fwrite (&param->h, sizeof (float), 1, file);
  fwrite (&param->factor, sizeof (float), 1, file);
}

void
TER_read_middisp_params (terrain_t * terrain, param_middisp_t * param, FILE * file)
{
  if (param) 
  {
    fread (&param->h, sizeof (float), 1, file);
    fread (&param->factor, sizeof (float), 1, file);
  }
  
  else fprintf(stderr,"ERROR: no parameters structure supplied\n");
}

void
TER_write_diamsquare_params (terrain_t * terrain, FILE * file)
{
  param_diamsquare_t *param = (param_diamsquare_t *) terrain->spec_param;

  fwrite (&param->h, sizeof (float), 1, file);
  fwrite (&param->factor, sizeof (float), 1, file);
  fwrite (&param->wrap, sizeof (int), 1, file);
}

void
TER_read_diamsquare_params (terrain_t * terrain, param_diamsquare_t * param, FILE * file)
{
  if (param) 
  {
    fread (&param->h, sizeof (float), 1, file);
    fread (&param->factor, sizeof (float), 1, file);
    fread (&param->wrap, sizeof (int), 1, file);
  }
  
  else fprintf(stderr,"ERROR: no parameters structure supplied\n");
}

int
TER_export_terrain (terrain_t * terrain, char *filename)
{
  FILE *file;
  int i=0;

  if (!terrain)
    return 0;

  if (!(file = fopen(filename, "wb")))
  {
    fprintf(stderr, "Error creating file: %s.\n", filename);
    return 0;
  }

  /* header pour verification de format lors d'une lecture future */
  fwrite (TERR_HEADER_VERIF, sizeof (char), 3, file);

  fwrite (&terrain->algo, sizeof (int), 1, file);
  fwrite (&terrain->size, sizeof (int), 1, file);
  
  fwrite (&terrain->color_scheme, sizeof (int), 1, file);

  /* parametres specifiques au type d'algo */
  switch (terrain->algo) {
  case TERR_FLAT:
    break;
  case TERR_MIDDISP:
    TER_write_middisp_params (terrain, file);
    break;
  case TERR_SPHERE:
    break;
  case TERR_HYPERB:
    TER_write_hyperb_params (terrain, file);
    break;
  case TERR_DIAM:
    TER_write_diamsquare_params (terrain, file);
    break;

  default:
    fprintf (stderr, "Warning: unknown algorithm\n");
    break;
  }

  /* terrain en lui meme */
  fwrite (terrain->heightmap_orig,
	  sizeof (float), terrain->size * terrain->size, file);
  fwrite (terrain->heightmap,
	  sizeof (float), terrain->size * terrain->size, file);
  fwrite (&terrain->max_height, sizeof (float), 1, file);
  
  /* filtres */
  for (i=0; i < NUM_EFFECTIVE_FILTERS; i++) {
    fwrite (&terrain->filters[i].name, sizeof (int), 1, file);
    fwrite (&terrain->filters[i].param, sizeof (float),1, file);
  }

  fclose(file);
  return 1;
}

int
TER_verify_terrain_file (FILE *file)
{
  char verification[3];
  if (!file) return 0;
    
  fread(verification, sizeof(char),3,file);
  if ((verification[0] != TERR_HEADER_VERIF[0]) 
      || (verification[1] != TERR_HEADER_VERIF[1])
      || (verification[2] != TERR_HEADER_VERIF[2]))   
    return 0;

  else return 1;
}

int
TER_read_algo (char *filename)
{
  FILE *file;
  int algo;
  
  if (!(file = fopen(filename, "rb")))
  {
    fprintf(stderr, "Error opening file: %s.\n", filename);
    return -1;
  }
  
  if (!TER_verify_terrain_file (file)) {
    fprintf(stderr, "Error: file is not a terrain\n");
    fclose(file);
    return -1;
  }
  
  fread (&algo, sizeof (int), 1, file);  
  fclose (file);
  
  return algo;
}

int
TER_import_terrain (terrain_t * terrain, void *param, char *filename)
{
  FILE *file;
  char verification[3];
  int i=0;
  
  if (!terrain || !param)
    return 0;

  if (!(file = fopen(filename, "rb")))
  {
    fprintf(stderr, "Error opening file: %s.\n", filename);
    return 0;
  }
  
  if (!TER_verify_terrain_file (file)) {
    fprintf(stderr, "Error: file is not a terrain\n");
    fclose(file);
    return 0;
  }

  fread (&terrain->algo, sizeof (int), 1, file);
  fread (&terrain->size, sizeof (int), 1, file);
  
  fread (&terrain->color_scheme, sizeof (int), 1, file);
  
  /* alloue la memoire pour les heightmaps */
  if (!init_heightmap (terrain))
    return 0;
  
  /* parametres specifiques au type d'algo */
  switch (terrain->algo) {
  case TERR_FLAT:
    break;
  case TERR_MIDDISP:
    TER_read_middisp_params (terrain, param, file);
    break;
  case TERR_SPHERE:
    break;
  case TERR_HYPERB:
    TER_read_hyperb_params (terrain, param, file);
    break;
  case TERR_DIAM:
    TER_read_diamsquare_params (terrain, param, file);
    break;

  default:
    fprintf (stderr, "Warning: unknown algorithm\n");
    break;
  }
  
  terrain->spec_param = (void *) param;

  /* terrain en lui meme */
  fread (terrain->heightmap_orig,
	  sizeof (float), terrain->size * terrain->size, file);
  fread (terrain->heightmap,
	  sizeof (float), terrain->size * terrain->size, file);
  fread (&terrain->max_height, sizeof (float), 1, file);
  
  /* filtres */
  for (i=0; i < NUM_EFFECTIVE_FILTERS; i++) {
    fread (&terrain->filters[i].name, sizeof (int), 1, file);
    fread (&terrain->filters[i].param, sizeof (float), 1, file);
  }
  
  fclose(file);
  return 1;
}

#endif /* DONT_HAVE_ZLIB */
