/* Fichier "tesselation.c"
 * fonctions qui transforment le tableau de hauteurs en display list
 * pour OpenGL
 * 
 * ATTENTION: ces fonctions doivent etre appeles APRES INITIALISATION
 *  d'OpenGL
 *
 * TODO: expliquer ici le repere choisi et tout l'bazar
 */

#include "projet.h"

static int edge_flag = 1;

void
create_skybox (terrain_t * terrain)
{
  GLuint textureid;
  vector3d_t v[3];
  triangle3d_t triangle;
  float *coord1, *coord2, *coord3;

  glEnable (GL_TEXTURE_2D);
  glGenTextures (1, &textureid);
  /*bind_bmp_texture (textureid, "texture.bmp"); */

  glBegin (GL_TRIANGLES);

  set_vector (&v[0], 64, 128, -64);
  set_vector (&v[1], -64, 128, -64);
  set_vector (&v[2], -64, 0, -64);
  set_triangle (&triangle, v[0], v[1], v[2]);

  coord1 = triangle.vertices[0].coord;
  coord2 = triangle.vertices[1].coord;
  coord3 = triangle.vertices[2].coord;
  /*glNormal3fv(triangle.normal.coord); */
  glTexCoord2f (1.0, 1.0);
  glVertex3fv (coord1);
  glTexCoord2f (0.0, 1.0);
  glVertex3fv (coord2);
  glTexCoord2f (0.0, 0.0);
  glVertex3fv (coord3);

  set_vector (&v[0], 64, 128, -64);
  set_vector (&v[1], -64, 0, -64);
  set_vector (&v[2], 64, 0, -64);
  set_triangle (&triangle, v[0], v[1], v[2]);

  coord1 = triangle.vertices[0].coord;
  coord2 = triangle.vertices[1].coord;
  coord3 = triangle.vertices[2].coord;
  glNormal3fv (triangle.normal.coord);
  glTexCoord2f (1.0, 1.0);
  glVertex3fv (coord1);
  glTexCoord2f (0.0, 0.0);
  glVertex3fv (coord2);
  glTexCoord2f (1.0, 0.0);
  glVertex3fv (coord3);

  glEnd ();
  glDisable (GL_TEXTURE_2D);

}

/* ======================================================
 * tessallate_terrain
 *
 * Description: cree la display list associee a un terrain,
 *  i.e. transforme le tableau de hauteurs en display list pour OpenGL
 *
 * TODO: expliquer ici le coup du 128x128
 *
 * Parametres: pointeur sur le terrain  traiter 
 * =====================================================
 */
void
tessallate_terrain (terrain_t * terrain)
{
  int i, j;
  float x, z;
  float inc;
  vector3d_t v[3];
  triangle3d_t triangle;
  int num_triangles = 0;

  glDeleteLists (terrain->displaylist, 1);
  terrain->displaylist = glGenLists (1);
  glNewList (terrain->displaylist, GL_COMPILE);

  /* quelle que soit la valeur de terrain->size, on effectue un rendu
     de taille 128x128 pour OpenGL.
     Par consquent, le parametre terrain->size represente la precision
     du rendu */
  inc = 128 / (float) terrain->size;

  glBegin (GL_TRIANGLES);

  /* le terrain est centr sur l'origine, dessin entre -64 et +64 */
  z = -64.0f;
  for (j = 0; j < terrain->size - 1; j++) {
    x = -64.0f;
    for (i = 0; i < terrain->size - 1; i++) {

      /* decoupage de chaque carr en 2 triangles */
      set_vector (&v[0], x + inc, get_heightmap_value (terrain, i + 1, j), z);
      set_vector (&v[1], x, get_heightmap_value (terrain, i, j), z);
      set_vector (&v[2], x, get_heightmap_value (terrain, i, j + 1), z + inc);

      set_triangle (&triangle, v[0], v[1], v[2]);
      draw_triangle (triangle, terrain->color_scheme);

      set_vector (&v[0], x + inc, get_heightmap_value (terrain, i + 1, j), z);
      set_vector (&v[1], x, get_heightmap_value (terrain, i, j + 1), z + inc);
      set_vector (&v[2], x + inc, get_heightmap_value (terrain, i + 1, j + 1),
		  z + inc);

      set_triangle (&triangle, v[0], v[1], v[2]);
      draw_triangle (triangle, terrain->color_scheme);

      num_triangles += 2;

      x += inc;
    }
    z += inc;
  }
  glEnd ();

  /*create_skybox(terrain); */

  g_print ("TESSELLATION: %d triangles\n", num_triangles);

  glEndList ();
  terrain->generated = 1;
}

/* ======================================================
 * draw_triangle
 *
 * Description: dessine un triangle et lui assigne une couleur en
 *  fonction de son altitude
 * Parametres: le triangle  dessiner 
 * =====================================================
 */
void
draw_triangle (triangle3d_t triangle, int colorscheme)
{
  float *coord1, *coord2, *coord3;

  /*GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 }; */
  GLfloat mat_specular[] = { 0.4, 0.4, 0.4, 0.4 };
  GLfloat mat_shininess[] = { 3.0 };
  GLfloat mat_amb_diff[] = { 1.0, 1.0, 1.0, 1.0 };

  glMaterialfv (GL_FRONT, GL_SPECULAR, mat_specular);
  glMaterialfv (GL_FRONT, GL_SHININESS, mat_shininess);
  glMaterialfv (GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_amb_diff);

  coord1 = triangle.vertices[0].coord;
  coord2 = triangle.vertices[1].coord;
  coord3 = triangle.vertices[2].coord;

  glNormal3fv (triangle.normal.coord);

  /* on place les 3 sommets */
  if (edge_flag)
    glEdgeFlag (TRUE);
  assign_color (mat_amb_diff, coord1[1], colorscheme);
  glMaterialfv (GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_amb_diff);
  glVertex3fv (coord1);

  if (!edge_flag)
    glEdgeFlag (TRUE);
  assign_color (mat_amb_diff, coord2[1], colorscheme);
  glMaterialfv (GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_amb_diff);
  glVertex3fv (coord2);

  if (edge_flag)
    glEdgeFlag (FALSE);
  assign_color (mat_amb_diff, coord3[1], colorscheme);
  glMaterialfv (GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_amb_diff);
  glVertex3fv (coord3);

  edge_flag = !edge_flag;
}

/* ======================================================
 * assign_color
 *
 * Description: assigne une couleur  un sommet en fonction de son altitude
 *  cette couleur est en fait un tableau  pass plus tard  OpenGL 
 *  via glMaterialfv (GL_AMBIENT_AND_DIFFUSE)
 * Parametres: tableau de GLfloat  remplir, et altitude du sommet 
 * =====================================================
 */
#define TERR_RED 0
#define TERR_GREEN 1
#define TERR_BLUE 2

void
assign_color_summer (GLfloat * amb_diff_array, float altitude)
{
  if (altitude < 0.01) {
    amb_diff_array[TERR_RED] = 0.0;
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.4;
  }

  else if (altitude < 0.4 * terrain->max_height) {
    amb_diff_array[TERR_RED] = 0.05;
    amb_diff_array[TERR_GREEN] = 0.5 * altitude / (0.4 * terrain->max_height);
    amb_diff_array[TERR_BLUE] = 0.05;
  }
  
  else if (altitude < 0.6 * terrain->max_height) {
    amb_diff_array[TERR_RED] = 0.2;
    amb_diff_array[TERR_GREEN] = 0.6 * altitude / (0.4 * terrain->max_height);
    amb_diff_array[TERR_BLUE] = 0.2;
  }

  else if (altitude < 0.85 * terrain->max_height) {
    amb_diff_array[TERR_RED] =
      0.3 + 0.4 * (0.6 * altitude) / (0.4 * terrain->max_height);
    amb_diff_array[TERR_GREEN] =
      0.3 + 0.4 * (0.6 * altitude) / (0.4 * terrain->max_height);
    amb_diff_array[TERR_BLUE] =
      0.3 + 0.4 * (0.6 * altitude) / (0.4 * terrain->max_height);
  }

  else {
    amb_diff_array[TERR_RED] = 1.0;
    amb_diff_array[TERR_GREEN] = 1.0;
    amb_diff_array[TERR_BLUE] = 1.0;
  }

  amb_diff_array[3] = 1.0;
}

/*void
assign_color_autumn (GLfloat * amb_diff_array, float altitude)
{
  if (altitude < 0.01) {
    amb_diff_array[TERR_RED] = 0.0;
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.4;
  }

  else if (altitude < 0.4 * terrain->max_height) {
    amb_diff_array[TERR_RED] = 0.8 * altitude / (0.4 * terrain->max_height);
    amb_diff_array[TERR_GREEN] = 0.8 * altitude / (0.4 * terrain->max_height);
    amb_diff_array[TERR_BLUE] = 0.1;
  }
  
  else if (altitude < 0.6 * terrain->max_height) {
    amb_diff_array[TERR_RED] = 0.2;
    amb_diff_array[TERR_GREEN] = 0.5 * altitude / (0.4 * terrain->max_height);
    amb_diff_array[TERR_BLUE] = 0.2;
  }

  else if (altitude < 0.8 * terrain->max_height) {
    amb_diff_array[TERR_RED] =
      0.3 + 0.4 * (0.6 * altitude) / (0.4 * terrain->max_height);
    amb_diff_array[TERR_GREEN] =
      0.3 + 0.4 * (0.6 * altitude) / (0.4 * terrain->max_height);
    amb_diff_array[TERR_BLUE] =
      0.3 + 0.4 * (0.6 * altitude) / (0.4 * terrain->max_height);
  }

  else {
    amb_diff_array[TERR_RED] = 1.0;
    amb_diff_array[TERR_GREEN] = 1.0;
    amb_diff_array[TERR_BLUE] = 1.0;
  }

  amb_diff_array[3] = 1.0;
}*/


void
assign_color_mars (GLfloat * amb_diff_array, float altitude)
{
  if (altitude < 0.01) {
    amb_diff_array[TERR_RED] = 0.1;
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.0;
  }

  else if (altitude < 0.4 * terrain->max_height) {
    amb_diff_array[TERR_RED] = 0.5 * altitude / (0.4 * terrain->max_height);
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.0;
  }

  else if (altitude < 0.8 * terrain->max_height) {
    amb_diff_array[TERR_RED] =
      0.3 + 0.3 * (0.6 * altitude) / (0.4 * terrain->max_height);
    amb_diff_array[TERR_GREEN] = 0.05;
    amb_diff_array[TERR_BLUE] = 0.05;
  }

  else {
    amb_diff_array[TERR_RED] = 0.5;
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.0;
  }

  amb_diff_array[3] = 1.0;
}

void
assign_color_venus (GLfloat * amb_diff_array, float altitude)
{
  if (altitude < 0.01) {
    amb_diff_array[TERR_RED] = 0.1;
    amb_diff_array[TERR_GREEN] = 0.2;
    amb_diff_array[TERR_BLUE] = 0.2;
  }

  else if (altitude < 0.4 * terrain->max_height) {
    amb_diff_array[TERR_RED] = 0.5 * altitude / (0.4 * terrain->max_height);
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.4;
  }

  else if (altitude < 0.8 * terrain->max_height) {
    amb_diff_array[TERR_RED] =
      0.3 + 0.3 * (0.6 * altitude) / (0.4 * terrain->max_height);
    amb_diff_array[TERR_GREEN] = 0.15;
    amb_diff_array[TERR_BLUE] = 0.25;
  }

  else {
    amb_diff_array[TERR_RED] = 0.5;
    amb_diff_array[TERR_GREEN] = 0.0;
    amb_diff_array[TERR_BLUE] = 0.5;
  }

  amb_diff_array[3] = 1.0;
}

void
assign_color (GLfloat * amb_diff_array, float altitude, int colorscheme)
{
  switch (colorscheme) {
  case TERR_COLOR_SUMMER:
    assign_color_summer (amb_diff_array, altitude);
    break;

  case TERR_COLOR_MARS:
    assign_color_mars (amb_diff_array, altitude);
    break;

  case TERR_COLOR_VENUS:
    assign_color_venus (amb_diff_array, altitude);
    break;

  default:
    assign_color_summer (amb_diff_array, altitude);
    break;
  }

}

/* ======================================================
 * create_test_square
 *
 * Description: permet de tester le rendu quand rien ne marche :-)
 *  cre une display list avec un terrain plat dedans
 * Parametres: pointeur sur terrain_t 
 * =====================================================
 */
void
create_test_square (terrain_t * terrain)
{

  vector3d_t v[3];
  triangle3d_t triangle;

  terrain->displaylist = glGenLists (1);
  glNewList (terrain->displaylist, GL_COMPILE);

  glBegin (GL_TRIANGLES);

  set_vector (&v[0], -64, 0, -64);
  set_vector (&v[1], -64, 0, 64);
  set_vector (&v[2], 64, 0, 64);

  set_triangle (&triangle, v[0], v[1], v[2]);
  draw_triangle (triangle, terrain->color_scheme);

  set_vector (&v[0], -64, 0, -64);
  set_vector (&v[1], 64, 0, 64);
  set_vector (&v[2], 64, 0, -64);

  set_triangle (&triangle, v[0], v[1], v[2]);
  draw_triangle (triangle, terrain->color_scheme);

  glEnd ();

  glEndList ();
}
